﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.PermutationEncoding;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  public enum ElementType { Complete, Diff };

  public class PermutationWrapper {
    public ElementType ElementType { get; set; }
    public Permutation Permutation { get; set; }
    public List<int> Diffs { get; set; }

    public PermutationWrapper(Permutation p) {
      ElementType = ElementType.Complete;
      Diffs = new List<int>();
      Permutation = p;
    }

    public Permutation GetPermutation() {
      if (ElementType == ElementType.Complete) {
        return Permutation;
      } else {
        Permutation result = (Permutation)Permutation.Clone(new Cloner());
        for (int i = 0; i < Diffs.Count; i++) {
          int index = Diffs[i] >> 16;
          int value = Diffs[i] & 0x0000FFFF;
          result[index] = value;
        }
        return result;
      }
    }

    public void StorePartialPermutation(Permutation original, Permutation newPermutation) {
      ElementType = ElementType.Diff;
      Diffs = new List<int>();
      Permutation = original;

      for (int i = 0; i < original.Length; i++) {
        if (original[i] != newPermutation[i]) {
          int diff = (i << 16) | newPermutation[i];
          Diffs.Add(diff);
        }
      }
    }

    public void StoreCompletePermutation(Permutation original) {
      ElementType = ElementType.Complete;
      Diffs = new List<int>();
      Permutation = original;
    }
  }
}
